use core::panic;
use std::{env, fs, io::Write, path::PathBuf, process};

use crate::{asm_gen, tokenizer};

pub fn compile(program: Vec<u8>, name: String) {
	let tokens = tokenizer::tokenize(program);
	let asm = asm_gen::generate(tokens);

	let out_path: PathBuf = [".", "out"].iter().collect();
	fs::DirBuilder::new()
		.recursive(true)
		.create(out_path)
		.unwrap();

	let path: PathBuf = [".", "out", &format!("{name}.asm")].iter().collect();
	let mut file = fs::File::create(path).unwrap();
	file.write_all(asm.as_bytes()).unwrap();

	if !is_program_in_path("nasm") {
		panic!("No nasm binary found in PATH");
	}

	process::Command::new("nasm")
		.args(["-felf64", &format!("./out/{name}.asm")])
		.output()
		.expect("failed compile step");

	if !is_program_in_path("ld") {
		panic!("No ld binary found in PATH");
	}

	process::Command::new("ld")
		.args([
			"-s",
			"-o",
			&format!("./out/{name}"),
			&format!("./out/{name}.o"),
		])
		.output()
		.expect("failed linking step");

	process::Command::new("rm")
		.args([
			"-r",
			&format!("./out/{name}.asm"),
			&format!("./out/{name}.o"),
		])
		.output()
		.expect("failed removing artifcats step");
}

fn is_program_in_path(program: &str) -> bool {
	if let Ok(path) = env::var("PATH") {
		for p in path.split(":") {
			let p_str = format!("{}/{}", p, program);
			if fs::metadata(p_str).is_ok() {
				return true;
			}
		}
	}
	false
}
